@@ -6,7 +6,7 @@ namespace codegen {
 
 TEST(codegen, unroll_async_id)
 {
-  test(R"(BEGIN { @i = 0; unroll(5) { printf("hi") } })", NAME);
+  test(R"(BEGIN { $i = 0; unroll(5) { $i += 1; } })", NAME);
 }
 
 } // namespace codegen
