@@ -39,7 +39,7 @@ public:
   {
     return resources.probes;
   }
-  std::vector<Probe> get_special_probes()
+  std::unordered_map<std::string, Probe> get_special_probes()
   {
     return resources.special_probes;
   }
@@ -81,11 +81,6 @@ public:
     mock_probe_matcher = dynamic_cast<MockProbeMatcher *>(probe_matcher_.get());
   }
 
-  int get_num_possible_cpus() const override
-  {
-    return 20;
-  }
-
   MockProbeMatcher *mock_probe_matcher;
 };
 
@@ -99,7 +94,7 @@ public:
     has_send_signal_ = std::make_optional<bool>(has_features);
     has_get_current_cgroup_id_ = std::make_optional<bool>(has_features);
     has_override_return_ = std::make_optional<bool>(has_features);
-    has_prog_kfunc_ = std::make_optional<bool>(has_features);
+    has_prog_fentry_ = std::make_optional<bool>(has_features);
     has_loop_ = std::make_optional<bool>(has_features);
     has_probe_read_kernel_ = std::make_optional<bool>(has_features);
     has_features_ = has_features;
@@ -109,7 +104,6 @@ public:
     has_uprobe_multi_ = std::make_optional<bool>(has_features);
     has_skb_output_ = std::make_optional<bool>(has_features);
     map_ringbuf_ = std::make_optional<bool>(has_features);
-    map_percpu_hash_ = std::make_optional<bool>(has_features);
     has_ktime_get_tai_ns_ = std::make_optional<bool>(has_features);
     has_get_func_ip_ = std::make_optional<bool>(has_features);
     has_jiffies64_ = std::make_optional<bool>(has_features);
@@ -121,6 +115,11 @@ public:
     has_loop_ = std::make_optional<bool>(has);
   }
 
+  void mock_missing_kernel_func(Kfunc kfunc)
+  {
+    available_kernel_funcs_.emplace(kfunc, false);
+  }
+
   bool has_features_;
 };
 
