@@ -6,7 +6,7 @@ import signal
 import sys
 import os
 import time
-from distutils.version import LooseVersion
+from looseversion import LooseVersion
 import re
 from functools import lru_cache
 
@@ -15,7 +15,6 @@ import cmake_vars
 BPFTRACE_BIN = os.environ["BPFTRACE_RUNTIME_TEST_EXECUTABLE"]
 COLOR_SETTING = os.environ.get("RUNTIME_TEST_COLOR", "auto")
 ATTACH_TIMEOUT = 10
-DEFAULT_TIMEOUT = 5
 
 
 OK_COLOR = '\033[92m'
@@ -57,6 +56,7 @@ class Runner(object):
     SKIP_FEATURE_REQUIREMENT_UNSATISFIED = 6
     SKIP_AOT_NOT_SUPPORTED = 7
     SKIP_KERNEL_VERSION_MAX = 8
+    SKIP_IN_SKIPLIST = 9
 
     @staticmethod
     def failed(status):
@@ -75,6 +75,7 @@ class Runner(object):
             Runner.SKIP_ENVIRONMENT_DISABLED,
             Runner.SKIP_FEATURE_REQUIREMENT_UNSATISFIED,
             Runner.SKIP_AOT_NOT_SUPPORTED,
+            Runner.SKIP_IN_SKIPLIST,
         ]
 
     @staticmethod
@@ -93,6 +94,8 @@ class Runner(object):
             return "disabled by environment variable"
         elif status == Runner.SKIP_AOT_NOT_SUPPORTED:
             return "aot does not yet support this"
+        elif status == Runner.SKIP_IN_SKIPLIST:
+            return "disabled by entry in skiplist file"
         else:
             raise ValueError("Invalid skip reason: %d" % status)
 
@@ -105,7 +108,7 @@ class Runner(object):
 
             return nsenter_prefix + ret
         else:  # PROG
-            use_json = "-q -f json" if test.expects[0].mode == "json" else ""
+            use_json = "-q -f json" if (len(test.expects) > 0 and test.expects[0].mode == "json") else ""
             escaped_prog = test.prog.replace("'", "'\\''")
             cmd = nsenter_prefix + "{} {} -e '{}'".format(BPFTRACE_BIN, use_json, escaped_prog)
             # We're only reusing PROG-directive tests for AOT tests
@@ -133,14 +136,14 @@ class Runner(object):
         bpffeature["loop"] = output.find("Loop support: yes") != -1
         bpffeature["probe_read_kernel"] = output.find("probe_read_kernel: yes") != -1
         bpffeature["btf"] = output.find("btf: yes") != -1
-        bpffeature["kfunc"] = output.find("kfunc: yes") != -1
+        bpffeature["fentry"] = output.find("fentry: yes") != -1
         bpffeature["dpath"] = output.find("dpath: yes") != -1
-        bpffeature["uprobe_refcount"] = \
-            output.find("uprobe refcount (depends on Build:bcc bpf_attach_uprobe refcount): yes") != -1
+        bpffeature["uprobe_refcount"] = output.find("uprobe refcount") != -1
         bpffeature["signal"] = output.find("send_signal: yes") != -1
         bpffeature["iter"] = output.find("iter: yes") != -1
         bpffeature["libpath_resolv"] = output.find("bcc library path resolution: yes") != -1
         bpffeature["dwarf"] = output.find("liblldb (DWARF support): yes") != -1
+        bpffeature["kernel_dwarf"] = output.find("Kernel DWARF: yes") != -1
         bpffeature["kprobe_multi"] = output.find("kprobe_multi: yes") != -1
         bpffeature["uprobe_multi"] = output.find("uprobe_multi: yes") != -1
         bpffeature["aot"] = cmake_vars.LIBBCC_BPF_CONTAINS_RUNTIME
@@ -388,7 +391,7 @@ class Runner(object):
                     attached = True
                     if test.has_exact_expect:
                         output = ""  # ignore earlier ouput
-                    signal.alarm(test.timeout or DEFAULT_TIMEOUT)
+                    signal.alarm(test.timeout)
                     if test.after:
                         after_cmd = get_pid_ns_cmd(test.after) if test.new_pidns else test.after
                         after = subprocess.Popen(after_cmd, shell=True,
@@ -463,26 +466,25 @@ class Runner(object):
         def print_befores_and_after_output():
             if len(befores_output) > 0:
                 for out in befores_output:
-                    out = out.encode("unicode_escape").decode("utf-8")
-                    print(f"\tBefore cmd output: {out}")
+                    print(f"\tBefore cmd output: {to_utf8(out)}")
             if after_output is not None:
-                out = after_output.encode("unicode_escape").decode("utf-8")
-                print(f"\tAfter cmd output: {out}")
+                print(f"\tAfter cmd output: {to_utf8(after_output)}")
 
-        if p and p.returncode != 0 and not test.will_fail and not timeout:
+        if '__BPFTRACE_NOTIFY_AOT_PORTABILITY_DISABLED' in to_utf8(output):
+            print(warn("[   SKIP   ] ") + "%s.%s" % (test.suite, test.name))
+            return Runner.SKIP_AOT_NOT_SUPPORTED
+
+        if p and p.returncode != test.return_code and not test.will_fail and not timeout:
             print(fail("[  FAILED  ] ") + "%s.%s" % (test.suite, test.name))
             print('\tCommand: ' + bpf_call)
             print('\tUnclean exit code: ' + str(p.returncode))
-            print('\tOutput: ' + output.encode("unicode_escape").decode("utf-8"))
+            print('\tOutput: ' + to_utf8(output))
             print_befores_and_after_output()
             return Runner.FAIL
 
         if result:
             print(ok("[       OK ] ") + "%s.%s" % (test.suite, test.name))
             return Runner.PASS
-        elif '__BPFTRACE_NOTIFY_AOT_PORTABILITY_DISABLED' in output:
-            print(warn("[   SKIP   ] ") + "%s.%s" % (test.suite, test.name))
-            return Runner.SKIP_AOT_NOT_SUPPORTED
         else:
             print(fail("[  FAILED  ] ") + "%s.%s" % (test.suite, test.name))
             print('\tCommand: ' + bpf_call)
